package reversi;

import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.JFrame;
import reversi.ai.AlphaBetaAI;
import reversi.ai.MinMaxAI;
import reversi.ai.RandomAI;
import reversi.ai.ReversiAI;
import reversi.model.Player;
import reversi.model.ReversiModel;
import reversi.view.ReversiControlPanel;
import reversi.view.ReversiController;
import reversi.view.ReversiPanel;


public class Main {

	public static void main(String[] args) {
		ReversiModel model = new ReversiModel();
		ReversiAI computers[] = {
				null,
				new RandomAI(Player.black),
				new RandomAI(Player.white),
				new MinMaxAI(Player.black, 3),
				new MinMaxAI(Player.white, 3),
				new MinMaxAI(Player.black, 5),
				new MinMaxAI(Player.white, 5),
				new AlphaBetaAI(Player.black, 5),
				new AlphaBetaAI(Player.white, 5)		
		};

		ReversiPanel view = new ReversiPanel(model);
		ReversiControlPanel control = new ReversiControlPanel(model, view, computers);
		ReversiController controller = new ReversiController(model, view);
		view.addMouseListener(controller);
		

		view.setPreferredSize(new Dimension(400, 400));		
		JFrame frame = new JFrame("Reversi");
		Box box = Box.createVerticalBox();
		box.add(control);
		box.add(view);
		frame.add(box);
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}

}
