package reversi.ai;
import java.util.List;

import reversi.model.Move;
import reversi.model.Player;
import reversi.model.ReversiBoard;

public class AlphaBetaAI extends ReversiAI {
	private int maxDepth;

	public AlphaBetaAI(Player p, int maxDepth){
		super(p);
		this.maxDepth = maxDepth;
	}
	
	@Override
	public String toString(){
		return super.toString() + "/ Alpha-Beta AI (" + maxDepth +"ǂ)";
	}
	
	@Override
	public MoveAndScore decideMove(ReversiBoard board) {
		if(board.getPossibleMoves(getTurn()).isEmpty()) return new MoveAndScore(null, 0);

		if(board.getStep() < 50){ // Ձ`Ղ͎̌₪̂Ő󂭓ǂ
			return alphabeta(board, maxDepth, Integer.MIN_VALUE, Integer.MAX_VALUE);
		}
		else{ // IՂ͎̌₪ȂȂ̂ŁA[ǂ
			return alphabeta(board, 12, Integer.MIN_VALUE, Integer.MAX_VALUE);
		}
	}
		
	private MoveAndScore alphabeta(ReversiBoard board, int depth, int alpha, int beta){
		if(depth == 0 || board.isFinished()){
			int score = ReversiEvaluator.evaluate(board, getTurn());
			return new MoveAndScore(null, score);
		}
		
		List<Move> moves = board.getPossibleMoves(board.getCurrentTurn());
		MoveAndScore result = new MoveAndScore(null, isMyTurn(board) ? alpha : beta);
		
		if(moves.isEmpty()){
			ReversiBoard child = new ReversiBoard();
			child.pass();
			return alphabeta(child, depth - 1, alpha, beta);
		}
		else if(board.getCurrentTurn() == getTurn()){
			for(Move m : moves){
				MoveAndScore mas = alphabeta(createChild(board, m), depth - 1, alpha, beta);
				if(mas.score > alpha){
					alpha = mas.score;
					result = new MoveAndScore(m, alpha);
				}
				if(alpha >= beta){
					return new MoveAndScore(null, beta);
				}
			}
			return result;
		}
		else{
			for(Move m : moves){
				MoveAndScore mas = alphabeta(createChild(board, m), depth - 1, alpha, beta);
				if(mas.score < beta){
					beta = mas.score;
					result = new MoveAndScore(m, beta);
				}
				if(alpha >= beta){
					return new MoveAndScore(null, alpha);
				}
			}
			return result;
		}
	}
}
