package reversi.ai;

import java.util.List;

import reversi.model.Move;
import reversi.model.Player;
import reversi.model.ReversiBoard;

public class MinMaxAI extends ReversiAI {
	private int maxDepth;

	public MinMaxAI(Player p, int maxDepth) {
		super(p);
		this.maxDepth = maxDepth;
	}
	
	@Override
	public String toString(){
		return super.toString() + "/ Min-Max AI (" + maxDepth +"ǂ)";
	}

	@Override
	public MoveAndScore decideMove(ReversiBoard board) {
		if(board.getPossibleMoves(getTurn()).isEmpty()) return new MoveAndScore(null, 0);

		if(board.getStep() < 50){ // Ձ`Ղ͎̌₪̂Ő󂭓ǂ
			return minmax(board, maxDepth);
		}
		else{ // IՂ͎̌₪ȂȂ̂ŁA[ǂ
			return minmax(board, 10);
		}
	}

	private MoveAndScore minmax(ReversiBoard board, int depth){
		if(depth == 0 || board.isFinished()){
			int score = ReversiEvaluator.evaluate(board, getTurn());
			return new MoveAndScore(null, score);
		}
		
		List<Move> moves = board.getPossibleMoves(board.getCurrentTurn());
		MoveAndScore selected = null;
		
		if(moves.isEmpty()){
			ReversiBoard child = new ReversiBoard();
			child.pass();
			return minmax(child, depth - 1);
		}
		else if(board.getCurrentTurn() == getTurn()){
			int max = Integer.MIN_VALUE;
			for(Move m : moves){
				MoveAndScore mas = minmax(createChild(board, m), depth - 1);
				if(mas.score > max){
					max = mas.score;
					selected = new MoveAndScore(m, mas.score);
				}
			}
			return selected;
		}
		else{
			int min = Integer.MAX_VALUE;
			for(Move m : moves){
				MoveAndScore mas = minmax(createChild(board, m), depth - 1);
				if(mas.score < min){
					min = mas.score;
					selected = new MoveAndScore(m, mas.score);
				}
			}
			return selected;
		}
	}
}
