package reversi.model;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ReversiBoard {
	public static final int BOARD_SIZE = 8;
	public static final int LAST = 60;
	private static final int[][] ds = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};

	private int step;
	private Player currentTurn;
	private Player[][] board;
	
	public ReversiBoard(){
		board = new Player[BOARD_SIZE][BOARD_SIZE];
		initBoard();
	}

	// Ƃč쐬
	public ReversiBoard(ReversiBoard model){
		board = new Player[BOARD_SIZE][BOARD_SIZE];
		for(int i = 0; i < ReversiBoard.BOARD_SIZE; i++){
			for(int j = 0; j < ReversiBoard.BOARD_SIZE; j++){
				currentTurn = model.getCurrentTurn();
				board[i][j] = model.get(i, j);
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////////
	// 擾郁\bh
	///////////////////////////////////////////////////////////////////////////

	public Player getCurrentTurn(){
		return currentTurn;
	}
	
	public Player get(int row, int column){
		return board[row][column];
	}
	
	// black/white ̐𐔂
	public int count(Player p){
		int n = 0;
		for(Player[] row : board){
			for(Player p2 : row){
				if(p == p2) n++;
			}
		}
		return n;
	}
	
	// ݉ڂ܂Ői񂾂
	public int getStep(){
		return step;
	}
	
	// IǂĂ邩ǂ (Ֆʂ܂Ă邩ǂuȂ)
	public boolean isFinished(){
		return getPossibleMoves(Player.black).size() + getPossibleMoves(Player.white).size() == 0;
	}
	
	// u邩ǂ
	public boolean isMovePossible(int row, int column, Player p){
		if(board[row][column] != Player.none) return false;
		for(int[] d : ds){
			if(check(row, column, d[0], d[1], p)) return true;
		}
		return false;
	}

	// uꏊׂẴXg
	public List<Move> getPossibleMoves(Player p){
		List<Move> moves = new ArrayList<Move>();
		for(int i = 0; i < BOARD_SIZE; i++){
			for(int j = 0; j < BOARD_SIZE; j++){
				if(isMovePossible(i, j, p)){
					moves.add(new Move(i, j));
				}
			}
		}
		return moves;
	}
	
	///////////////////////////////////////////////////////////////////////////
	// ύX郁\bh
	///////////////////////////////////////////////////////////////////////////
	
	public void initBoard(){
		currentTurn = Player.black;
		step = 0;
		for(Player[] row : board){
			Arrays.fill(row, Player.none);
		}
		board[3][3] = Player.black;
		board[3][4] = Player.white;
		board[4][3] = Player.white;
		board[4][4] = Player.black;
	}
	
	// u
	public boolean move(Move m){
		return move(m.getRow(), m.getColumn());
	}
	
	// u
	public boolean move(int row, int column){
		if(board[row][column] != Player.none) return false;

		boolean result = false;
		for(int[] d : ds){
			result |= flip(row, column, d[0], d[1], currentTurn);
		}
		
		if(result){
			board[row][column] = currentTurn;
			currentTurn = currentTurn == Player.black ? Player.white : Player.black;
			step++;
		}
		return result;
	}

	// pXiuƂ̓pXłȂƂɒӁj
	public boolean pass(){
		if(getPossibleMoves(currentTurn).size() > 0) return false;
		else{
			currentTurn = currentTurn == Player.black ? Player.white : Player.black;
			return true;
		}
	}
	
	///////////////////////////////////////////////////////////////////////////
	// Private \bh
	///////////////////////////////////////////////////////////////////////////
		
	private boolean isInBoard(int row, int column){
		return row >= 0 && row < BOARD_SIZE && column >= 0 && column < BOARD_SIZE;
	}
		
	// (r, c)  (dr, dc) ɂЂԂ邩ׂBЂԂ true ԂB
	private boolean check(int r, int c, int dr, int dc, Player p){
		int i = r + dr;
		int j = c + dc;
		boolean flag = false;

		while(isInBoard(i, j)){
			if(board[i][j] == Player.none) return false;
			else if(board[i][j] == p) return flag;
			else flag = true;
			i += dr;
			j += dc;			
		}

		return false;
	}
	
	// (r, c)  (dr, dc) ɂЂԂBPłЂԂ true ԂB
	private boolean flip(int r, int c, int dr, int dc, Player p){
		if(!check(r, c, dr, dc, p)) return false;
		
		int i = r + dr;
		int j = c + dc;
		
		while(board[i][j] != p){
			board[i][j] = p;
			i += dr;
			j += dc;
		}
		
		return true;
	}
}
