package reversi.view;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import reversi.ai.MoveAndScore;
import reversi.ai.ReversiAI;
import reversi.model.Move;
import reversi.model.ReversiBoard;
import reversi.model.ReversiModel;
import reversi.model.ReversiModelListener;

/** AI I邽߂ Component **/
public class ReversiControlPanel extends JPanel implements ReversiModelListener, ItemListener {
	private ReversiModel model;
	private ReversiPanel view;
	private ReversiAI[] computers;

	private JComboBox computerSelecter;
	private JLabel label;

	public ReversiControlPanel(ReversiModel model, ReversiPanel view, ReversiAI[] computers) {
		super();
		this.model = model;
		this.view = view;
		model.addReversiModelListener(this);
		this.computers = computers;
		initComponent();
	}
	
	private void initComponent(){
		computerSelecter = new JComboBox(computers);
		computerSelecter.addItemListener(this);
		add(computerSelecter);
		label = new JLabel();
		add(label);
	}
	
	private ReversiAI getAI(){
		return (ReversiAI) computerSelecter.getSelectedItem();
	}
	
	private boolean isComputerTurn(){
		return getAI() != null ? model.getCurrentTurn() == getAI().getTurn() : false;
	}
	
	private void playComputer(){
		MoveAndScore moveAndScore = getAI().decideMove(new ReversiBoard(model));
		Move m = moveAndScore.m;
		if(m == null) model.pass();
		else{
			model.move(m);
			label.setText("]l:" +  moveAndScore.score);
		}
	}

	@Override
	public void changed(ReversiModel model) {
		if(!model.isFinished() && isComputerTurn()) playComputer();
	}

	@Override
	public void itemStateChanged(ItemEvent arg0) {
		model.initBoard();
		if(getAI() != null) view.setComputerTurn(getAI().getTurn());
		else view.setComputerTurn(null);

		if(isComputerTurn()){
			playComputer();
		}
	}
}
