package reversi.view;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.LayoutManager;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import reversi.model.Player;
import reversi.model.ReversiBoard;
import reversi.model.ReversiModel;
import reversi.model.ReversiModelListener;

/** o[V̔Ֆʂ`悷NX **/
public class ReversiPanel extends JPanel implements ReversiModelListener {
	private ReversiModel model;
	private Player computerTurn;

	public ReversiPanel(ReversiModel model) {
		this.model = model;
		model.addReversiModelListener(this);
		this.setBackground(Color.green);
		setComputerTurn(Player.none);
	}
	
	public void setComputerTurn(Player turn){
		computerTurn = turn;
	}
		
	public void moveByLocation(int x, int y){
		if(model.isFinished()) model.initBoard();
		if(model.getCurrentTurn() == computerTurn) return;
		
		int row = x * ReversiBoard.BOARD_SIZE / getHeight();
		int column = y * ReversiBoard.BOARD_SIZE / getWidth();
		if(!model.move(row, column)){
			if(model.getPossibleMoves(model.getCurrentTurn()).isEmpty()){ // uꏊȂƂ̓pX
				model.pass();
			}
		}
	}
			
	private void doFinish(){
		int b = model.count(Player.black);
		int w = model.count(Player.white);
		int d = Math.abs(b - w);
		String message = b == w ? "ł" : d + "΍" + (b > w ? "̏ł" : "̏ł");
		JOptionPane.showMessageDialog(this, message);
	}
	
	public void changed(ReversiModel model){
		repaint();
		if(model.isFinished()) doFinish();
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Player turn = model.getCurrentTurn();
		int dw = getWidth() / ReversiBoard.BOARD_SIZE;
		int dh = getHeight() / ReversiBoard.BOARD_SIZE;
		
		g.setColor(Color.black);
		for(int i = 0; i < ReversiBoard.BOARD_SIZE; i++){ // 
			g.drawLine(0, dh * i, getWidth(), dh * i);
		}

		for(int j = 0; j < ReversiBoard.BOARD_SIZE; j++){ // c
			g.drawLine(dw * j, 0, dw * j, getWidth());
		}
		
		for(int i = 0; i < ReversiBoard.BOARD_SIZE; i++){
			for(int j = 0; j < ReversiBoard.BOARD_SIZE; j++){
				Player p = model.get(i, j);
				int x = dw * i;
				int y = dh * j;
				if(model.isMovePossible(i, j, turn)){
					g.setColor(turn.getColor());
					g.drawOval(x, y, dw, dh);
				}
				switch(p){
				case black:
				case white:
					g.setColor(p.getColor());
					g.fillOval(x, y, dw, dh);
					break;
				}
			}
		}
	}
}
